@@ -21,6 +21,9 @@
 #include "base_db.h"
 #include "application.h" //App_Glom.
 #include "appstate.h"
+#include "standard_table_prefs_fields.h"
+#include "document/document_glom.h"
+#include "data_structure/glomconversions.h"
 #include <glibmm/i18n.h>
 //#include <libgnomeui/gnome-app-helper.h>
 
@@ -28,26 +31,27 @@
 
 Base_DB::Base_DB()
 {
-  m_pDocument = 0;
+  //m_pDocument = 0;
 }
 
 Base_DB::~Base_DB()
 {
 }
 
-void Base_DB::init_db_details()
+bool Base_DB::init_db_details()
 {
-  fill_from_database();
+  return fill_from_database();
 }
 
-void Base_DB::refresh_db_details()
+bool Base_DB::refresh_data_from_database()
 {
-  fill_from_database();
+  return fill_from_database();
 }
 
-void Base_DB::fill_from_database()
+bool Base_DB::fill_from_database()
 {
   //m_AddDel.remove_all();
+  return true;
 }
 
 void Base_DB::fill_end()
@@ -133,13 +137,17 @@ Glib::RefPtr<Gnome::Gda::DataModel> Base_DB::Query_execute(const Glib::ustring&
       handle_error();
     }
   }
+  else
+  {
+    g_warning("Base_DB::Query_execute(): No connection yet.");
+  }
 
   return result;
 }
 
 void Base_DB::load_from_document()
 {
-  if(m_pDocument)
+  if(get_document())
   {
     fill_from_database(); //virtual.
 
@@ -148,8 +156,18 @@ void Base_DB::load_from_document()
   }
 }
 
+bool Base_DB::get_table_exists_in_database(const Glib::ustring& table_name) const
+{
+  //TODO_Performance
 
-Base_DB::type_vecStrings Base_DB::get_table_names()
+  type_vecStrings tables = get_table_names();
+  type_vecStrings::const_iterator iterFind = std::find(tables.begin(), tables.end(), table_name);
+  bool result = (iterFind != tables.end());
+
+  return result;
+}
+
+Base_DB::type_vecStrings Base_DB::get_table_names() const
 {
   type_vecStrings result;
 
@@ -242,12 +260,13 @@ void Base_DB::set_document(Document_Glom* pDocument)
   View_Composite_Glom::set_document(pDocument);
 
   //Connect to a signal that is only on the derived document class:
-  if(m_pDocument)
+  Document_Glom* document = get_document();
+  if(document)
   {
-    m_pDocument->signal_userlevel_changed().connect( sigc::mem_fun(*this, &Base_DB::on_userlevel_changed) );
+    document->signal_userlevel_changed().connect( sigc::mem_fun(*this, &Base_DB::on_userlevel_changed) );
 
     //Show the appropriate UI for the user level that is specified by this new document:
-    on_userlevel_changed(m_pDocument->get_userlevel());
+    on_userlevel_changed(document->get_userlevel());
   }
 
 
@@ -307,11 +326,11 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table_from_database(const Glib::
 
     if(!data_model_fields)
     {
-      std::cerr << "Base_DB_Table_Definition::fill_fields(): libgda reported empty fields schema data_model for the table." << std::endl;
+      std::cerr << "Base_DB::get_fields_for_table_from_database(): libgda reported empty fields schema data_model for the table." << std::endl;
     } 
     else if(data_model_fields->get_n_columns() == 0)
     {
-      std::cerr << "Base_DB_Table_Definition::fill_fields(): libgda reported 0 fields for the table." << std::endl;
+      std::cerr << "BBase_DB::get_fields_for_table_from_database(): libgda reported 0 fields for the table." << std::endl;
     }
     else if(data_model_fields->get_n_rows() == 0)
     {
@@ -634,6 +653,16 @@ Privileges Base_DB::get_table_privileges(const Glib::ustring& group_name, const
 {
   Privileges result;
 
+  if(group_name == GLOM_STANDARD_GROUP_NAME_DEVELOPER)
+  {
+    //Always give developers full access:
+    result.m_view = true;
+    result.m_edit = true;
+    result.m_create = true;
+    result.m_delete = true;
+    return result;
+  }
+
   //Get the permissions:
   Glib::ustring strQuery = "SELECT pg_class.relacl FROM pg_class WHERE pg_class.relname = '" + table_name + "'";
   Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute(strQuery);
@@ -699,9 +728,108 @@ Privileges Base_DB::get_table_privileges(const Glib::ustring& group_name, const
     }
   }
 
+  g_warning("get_table_privileges(group_name=%s, table_name=%s) returning: %d", group_name.c_str(), table_name.c_str(), result.m_create);
   return result;
 }
 
+void Base_DB::add_standard_tables() const
+{
+  //Name, address, etc:
+  if(!get_table_exists_in_database(GLOM_STANDARD_TABLE_PREFS_TABLE_NAME))
+  {
+    TableInfo prefs_table_info;
+    prefs_table_info.m_name = GLOM_STANDARD_TABLE_PREFS_TABLE_NAME;
+    prefs_table_info.m_hidden = true;
+
+    Document_Glom::type_vecFields pref_fields;
+
+    Field primary_key; //It's not used, because there's only one record, but we must have one.
+    primary_key.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ID);
+    primary_key.set_glom_type(Field::TYPE_NUMERIC);
+    pref_fields.push_back(primary_key);
+
+    Field field_name;
+    field_name.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_NAME);
+    field_name.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_name);
+
+    Field field_org_name;
+    field_org_name.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_NAME);
+    field_org_name.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_name);
+
+    Field field_org_address_street;
+    field_org_address_street.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET);
+    field_org_address_street.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_street);
+
+    Field field_org_address_street2;
+    field_org_address_street2.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET2);
+    field_org_address_street2.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_street2);
+
+    Field field_org_address_town;
+    field_org_address_town.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_TOWN);
+    field_org_address_town.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_town);
+
+    Field field_org_address_county;
+    field_org_address_county.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTY);
+    field_org_address_county.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_county);
+
+    Field field_org_address_country;
+    field_org_address_country.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTRY);
+    field_org_address_country.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_country);
+
+    Field field_org_address_postcode;
+    field_org_address_postcode.set_name(GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_POSTCODE);
+    field_org_address_postcode.set_glom_type(Field::TYPE_TEXT);
+    pref_fields.push_back(field_org_address_postcode);
+
+    bool test = create_table(prefs_table_info, pref_fields);
+
+    if(test)
+    {
+      //Add the single record:
+      Query_execute("INSERT INTO " GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "(" GLOM_STANDARD_TABLE_PREFS_FIELD_ID ") VALUES (1)");
+    }
+  }
+
+  //Auto-increment next values:
+  if(!get_table_exists_in_database(GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME))
+  {
+    TableInfo table_info;
+    table_info.m_name = GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME;
+    table_info.m_hidden = true;
+
+    Document_Glom::type_vecFields fields;
+
+    Field primary_key; //It's not used, because there's only one record, but we must have one.
+    primary_key.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_ID);
+    primary_key.set_glom_type(Field::TYPE_NUMERIC);
+    fields.push_back(primary_key);
+
+    Field field_table_name;
+    field_table_name.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME);
+    field_table_name.set_glom_type(Field::TYPE_TEXT);
+    fields.push_back(field_table_name);
+
+    Field field_field_name;
+    field_field_name.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME);
+    field_field_name.set_glom_type(Field::TYPE_TEXT);
+    fields.push_back(field_field_name);
+
+    Field field_next_value;
+    field_next_value.set_name(GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_NEXT_VALUE);
+    field_next_value.set_glom_type(Field::TYPE_TEXT);
+    fields.push_back(field_next_value);
+
+    create_table(table_info, fields);
+  }
+}
+
 void Base_DB::add_standard_groups()
 {
   //Add the glom_developer group if it does not exist:
@@ -712,13 +840,17 @@ void Base_DB::add_standard_groups()
   if(iterFind == vecGroups.end())
   {
     Query_execute("CREATE GROUP " GLOM_STANDARD_GROUP_NAME_DEVELOPER);
-    Privileges priv_ignored;
+    Privileges priv_devs;
+    priv_devs.m_view = true;
+    priv_devs.m_edit = true;
+    priv_devs.m_create = true;
+    priv_devs.m_delete = true;
 
     Document_Glom::type_listTableInfo table_list = get_document()->get_tables();
 
     for(Document_Glom::type_listTableInfo::const_iterator iter = table_list.begin(); iter != table_list.end(); ++iter)
     {
-      set_table_privileges(devgroup, iter->m_name, priv_ignored, true /* developer privileges */);
+      set_table_privileges(devgroup, iter->m_name, priv_devs, true /* developer privileges */);
     }
 
     //Make sure that it is in the database too:
@@ -729,6 +861,68 @@ void Base_DB::add_standard_groups()
   }
 }
 
+Gnome::Gda::Value Base_DB::auto_increment_insert_first_if_necessary(const Glib::ustring& table_name, const Glib::ustring& field_name)
+{
+  Gnome::Gda::Value value;
+
+  const Glib::ustring sql_query = "SELECT " GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME ".next_value FROM " GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME
+   " WHERE " GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME " = '" + table_name + "' AND "
+             GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME " = '" + field_name + "'";
+
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+  if(!datamodel || (datamodel->get_n_rows() == 0))
+  {
+    //Start with zero:
+
+    //Insert the row if it's not there.
+    const Glib::ustring sql_query = "INSERT INTO " GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME " ("
+      GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME ", " GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME ", " GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_NEXT_VALUE
+      ") VALUES ('" + table_name + "', '" + field_name + "', 0)";
+
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+    if(!datamodel)
+    {
+      g_warning("Base_DB::auto_increment_insert_first_if_necessary(): INSERT of new row failed.");
+    }
+
+    //GdaNumeric is a pain, so we take a short-cut:
+    bool success = false;
+    value = GlomConversions::parse_value(Field::TYPE_NUMERIC, "0", success, true /* iso_format */);
+  }
+  else
+  {
+    //Return the value so that a calling function does not need to do a second SELECT.
+    value = datamodel->get_value_at(0, 0);
+  }
+
+  return value;
+}
+
+Gnome::Gda::Value Base_DB::get_next_auto_increment_value(const Glib::ustring& table_name, const Glib::ustring& field_name)
+{
+  const Gnome::Gda::Value result = auto_increment_insert_first_if_necessary(table_name, field_name);
+  long num_result = 0;
+  num_result = util_decimal_from_string(result.to_string());
+
+
+  //Increment the next_value:
+  ++num_result;
+  const Gnome::Gda::Value next_value = GlomConversions::parse_value(num_result);
+
+  const Glib::ustring sql_query = "UPDATE " GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME " SET "
+      GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_NEXT_VALUE " = " + next_value.to_string() +
+      " WHERE " GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME " = '" + table_name + "' AND "
+                GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME " = '" + field_name + "'";
+
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+  if(!datamodel)
+  {
+    g_warning("Base_DB::get_next_auto_increment_value(): Increment failed.");
+  }
+
+  return result;
+}
+
 
 Glib::ustring Base_DB::string_trim(const Glib::ustring& str, const Glib::ustring& to_remove)
 {
@@ -876,3 +1070,120 @@ Privileges Base_DB::get_current_privs(const Glib::ustring& table_name) const
 
   return result;
 }
+
+SystemPrefs Base_DB::get_database_preferences() const
+{
+  //if(get_userlevel() == AppState::USERLEVEL_DEVELOPER)
+  //  add_standard_tables();
+
+  SystemPrefs result;
+
+  const Glib::ustring sql_query = "SELECT "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_NAME ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_NAME ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET2 ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_TOWN ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTY ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTRY ", "
+      GLOM_STANDARD_TABLE_PREFS_TABLE_NAME "." GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_POSTCODE
+      " FROM " GLOM_STANDARD_TABLE_PREFS_TABLE_NAME;
+
+  try
+  {
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+    if(datamodel && (datamodel->get_n_rows() != 0))
+    {
+      result.m_name = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(0, 0));
+      result.m_org_name = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(1, 0));
+      result.m_org_address_street = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(2, 0));
+      result.m_org_address_street2 = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(3, 0));
+      result.m_org_address_town = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(4, 0));
+      result.m_org_address_county = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(5, 0));
+      result.m_org_address_country = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(6, 0));
+      result.m_org_address_postcode = GlomConversions::get_text_for_gda_value(Field::TYPE_TEXT, datamodel->get_value_at(7, 0));      }
+  }
+  catch(const std::exception& ex)
+  {
+    std::cerr << "Base_DB::get_database_preferences(): exception: " << ex.what() << std::endl;
+  }
+
+  return result;
+}
+
+void Base_DB::set_database_preferences(const SystemPrefs& prefs)
+{
+  if(get_userlevel() == AppState::USERLEVEL_DEVELOPER)
+    add_standard_tables();
+
+  const Glib::ustring sql_query = "UPDATE " GLOM_STANDARD_TABLE_PREFS_TABLE_NAME " SET "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_NAME " = '" + prefs.m_name + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_NAME " = '" + prefs.m_org_name + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET " = '" + prefs.m_org_address_street + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_STREET2 " = '" + prefs.m_org_address_street2 + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_TOWN " = '" + prefs.m_org_address_town + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTY " = '" + prefs.m_org_address_county + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_COUNTRY " = '" + prefs.m_org_address_country + "', "
+      GLOM_STANDARD_TABLE_PREFS_FIELD_ORG_ADDRESS_POSTCODE " = '" + prefs.m_org_address_postcode + "'"
+      " WHERE " GLOM_STANDARD_TABLE_PREFS_FIELD_ID " = 1";
+
+    try
+    {
+      Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+    }
+    catch(const std::exception& ex)
+    {
+      std::cerr << "Base_DB::set_database_preferences(): exception: " << ex.what() << std::endl;
+    }
+}
+
+bool Base_DB::create_table(const TableInfo& table_info, const Document_Glom::type_vecFields& fields) const
+{
+  bool table_creation_succeeded = false;
+
+  //Create SQL to describe all fields in this table:
+  Glib::ustring sql_fields;
+  for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
+  {
+    //Create SQL to describe this field:
+    Field field = *iter;
+
+    //The field has no gda type, so we set that:
+    //This usually comes from the database, but that's a bit strange.
+    Gnome::Gda::FieldAttributes info = field.get_field_info();
+    info.set_gdatype( Field::get_gda_type_for_glom_type(field.get_glom_type()) );
+    field.set_field_info(info);
+
+    Glib::ustring sql_field_description = field.get_name() + " " + field.get_sql_type();
+
+    if(field.get_primary_key())
+      sql_field_description += " NOT NULL  PRIMARY KEY";
+
+    //Append it:
+    if(!sql_fields.empty())
+      sql_fields += ", ";
+
+    sql_fields += sql_field_description;
+  }
+
+  if(sql_fields.empty())
+  {
+    g_warning("Base_Db::create_table::create_table(): sql_fields is empty.");
+  }
+
+  //Actually create the table
+  try
+  {
+    Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute( "CREATE TABLE \"" + table_info.get_name() + "\" (" + sql_fields + ")" );
+    if(!data_model)
+      table_creation_succeeded = false;
+    else
+      table_creation_succeeded = true;
+  }
+  catch(const ExceptionConnection& ex)
+  {
+    table_creation_succeeded = false;
+  }
+
+  return table_creation_succeeded;
+}
