@@ -27,6 +27,7 @@
 #include "connectionpool.h"
 #include "appstate.h"
 #include "data_structure/privileges.h"
+#include "data_structure/system_prefs.h"
 #include "bakery/View/View.h"
 #include <bakery/Utilities/BusyCursor.h>
 
@@ -40,10 +41,10 @@ public:
   virtual ~Base_DB();
 
   /// Specify the structure of what will be shown, and fill it.
-  virtual void init_db_details();
+  virtual bool init_db_details();
 
   /// Specify what actual data will be shown:
-  virtual void refresh_data_from_database();
+  virtual bool refresh_data_from_database();
 
   /** Returns whether we are in developer mode.
    * Some functionality will be deactivated when not in developer mode.
@@ -67,11 +68,17 @@ public:
 
   //This is const because const means not changing this instance, not whether we change the database.
   virtual Glib::RefPtr<Gnome::Gda::DataModel> Query_execute(const Glib::ustring& strQuery) const;
+
   void add_standard_groups();
+  void add_standard_tables() const;
+
+  bool create_table(const TableInfo& table_info, const Document_Glom::type_vecFields& fields) const;
 
 protected:
   typedef std::vector<Glib::ustring> type_vecStrings;
-  type_vecStrings get_table_names();
+  type_vecStrings get_table_names() const;
+
+  bool get_table_exists_in_database(const Glib::ustring& table_name) const;
 
   type_vecFields get_fields_for_table(const Glib::ustring& table_name) const;
 
@@ -85,7 +92,17 @@ protected:
   bool get_user_is_in_group(const Glib::ustring& user, const Glib::ustring& group) const;
   Privileges get_current_privs(const Glib::ustring& table_name) const;
 
-  virtual void fill_from_database();
+  SystemPrefs get_database_preferences() const;
+  void set_database_preferences(const SystemPrefs& prefs);
+
+  Gnome::Gda::Value auto_increment_insert_first_if_necessary(const Glib::ustring& table_name, const Glib::ustring& field_name);
+
+  /** Get the next auto-increment value for this primary key, from the glom system table.
+   * Add a row for this field in the system table if it does not exist already.
+   */
+  Gnome::Gda::Value get_next_auto_increment_value(const Glib::ustring& table_name, const Glib::ustring& field_name);
+
+  virtual bool fill_from_database();
   virtual void fill_end(); //Call this from the end of fill_from_database() overrides.
 
   virtual void on_userlevel_changed(AppState::userlevels userlevel);
