@@ -22,10 +22,12 @@
 #include "application.h"
 #include "appstate.h"
 #include "mode_design/users/dialog_groups_list.h"
+#include "dialog_database_preferences.h"
 #include <glibmm/i18n.h>
 
 Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
 : PlaceHolder(cobject, refGlade),
+  m_pLabel_Name(0),
   m_pLabel_Table(0),
   m_pLabel_Mode(0),
   m_pLabel_userlevel(0),
@@ -38,6 +40,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   m_pDialogConnectionFailed(0)
 {
   //Load widgets from glade file:
+  refGlade->get_widget("label_name", m_pLabel_Name);
   refGlade->get_widget("label_table_name", m_pLabel_Table);
   refGlade->get_widget("label_mode", m_pLabel_Mode);
   refGlade->get_widget("label_user_level", m_pLabel_userlevel);
@@ -123,8 +126,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   //Connect signals:
   m_pBox_Tables->signal_selected.connect(sigc::mem_fun(*this, &Frame_Glom::on_box_tables_selected));
 
-  
-  m_Notebook_Find.signal_find.connect(sigc::mem_fun(*this, &Frame_Glom::on_Notebook_Find));
+  m_Notebook_Find.signal_find_criteria.connect(sigc::mem_fun(*this, &Frame_Glom::on_notebook_find_criteria));
 
   //Fill Composite View:
   //This means that set_document and load/save are delegated to these children:
@@ -136,6 +138,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   add_view(&m_Notebook_Find); //Also a composite view.
 
   on_userlevel_changed(AppState::USERLEVEL_OPERATOR); //A default to show before a document is created or loaded.
+
   show_all();
 }
 
@@ -211,11 +214,11 @@ void Frame_Glom::set_mode_widget(Gtk::Widget& widget)
     widget.show();
 
     //Show help text:
-    Notebook_Glom* pNotebook = dynamic_cast<Notebook_Glom*>(&widget);
-    if(pNotebook)
-    {
-      pNotebook->show_hint();
-    }
+    //Notebook_Glom* pNotebook = dynamic_cast<Notebook_Glom*>(&widget);
+    //if(pNotebook)
+   // {
+   //   pNotebook->show_hint();
+   // }
   }
 }
 
@@ -469,12 +472,31 @@ void Frame_Glom::show_ok_dialog(const Glib::ustring& title, const Glib::ustring&
   dialog.run();
 }
 
-void Frame_Glom::on_Notebook_Find(Glib::ustring strWhereClause)
+void Frame_Glom::on_notebook_find_criteria(const Glib::ustring& strWhereClause)
 {
-  on_menu_Mode_Data();
+  //on_menu_Mode_Data();
 
-  m_Notebook_Data.init_db_details(m_strTableName, strWhereClause);
-  m_Notebook_Data.select_page_for_find_results();
+  App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
+  if(pApp)
+  {
+    pApp->set_mode_data();
+
+    bool records_found = m_Notebook_Data.init_db_details(m_strTableName, strWhereClause);
+    m_Notebook_Data.select_page_for_find_results();
+
+    if(!records_found)
+    {
+      Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("No Records Found")), true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE);
+      dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+      dialog.add_button(_("New Find"), Gtk::RESPONSE_OK);
+      dialog.set_secondary_text(_("Your find criteria did not match any records in the table."));
+      dialog.set_transient_for(*(get_app_window()));
+      bool find_again = dialog.run();
+
+      if(find_again)
+        pApp->set_mode_find();
+    }
+  }
 }
 
 void Frame_Glom::on_userlevel_changed(AppState::userlevels userlevel)
@@ -603,6 +625,23 @@ void Frame_Glom::set_document(Document_Glom* pDocument)
   }
 }
 
+void Frame_Glom::show_system_name()
+{
+  const SystemPrefs prefs = get_database_preferences();
+  const Glib::ustring org = prefs.m_org_name;
+  const Glib::ustring name = prefs.m_name;
+
+  Glib::ustring system_name = org;
+  if(!system_name.empty() && !name.empty())
+    system_name += ": ";
+
+  system_name += name;
+
+  m_pLabel_Name->set_text ( Bakery::App_Gtk::util_bold_message(system_name) );
+  m_pLabel_Name->set_use_markup();
+}
+
+
 void Frame_Glom::load_from_document()
 {
   Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
@@ -613,6 +652,34 @@ void Frame_Glom::load_from_document()
   }
 }
 
+void Frame_Glom::on_menu_developer_database_preferences()
+{
+  Dialog_Database_Preferences* dialog = 0;
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_database_preferences");
+    refXml->get_widget_derived("dialog_database_preferences", dialog);
+    if(dialog)
+    {
+      dialog->set_transient_for(*(get_app_window()));
+      add_view(dialog);
+      dialog->load_from_document();
+
+      dialog->run();
+
+      remove_view(dialog);
+      delete dialog;
+
+      show_system_name(); //In case it has changed.
+    }
+  }
+
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+}
+
 void Frame_Glom::on_menu_developer_fields()
 {
   //Check that there is a table to show:
