@@ -63,6 +63,7 @@ public:
   virtual void on_menu_Navigate_Table();
   virtual void do_menu_Navigate_Table(bool open_default = false);
 
+  virtual void on_menu_developer_database_preferences();
   virtual void on_menu_developer_fields();
   virtual void on_menu_developer_relationships();
   virtual void on_menu_developer_users();     
@@ -74,6 +75,8 @@ public:
   virtual void set_document(Document_Glom* pDocument); //View override
   virtual void load_from_document(); //View override
 
+  void show_system_name();
+
   enum enumModes
   {
     MODE_None, //at the start.
@@ -108,13 +111,14 @@ protected:
   virtual void alert_no_table();
 
   //Signal handlers:
-  virtual void on_Notebook_Find(Glib::ustring strWhereClause);
+  virtual void on_notebook_find_criteria(const Glib::ustring& strWhereClause);
   virtual void on_userlevel_changed(AppState::userlevels userlevel);
 
   //Member data:
   Glib::ustring m_strTableName;
 
   //Child widgets:
+  Gtk::Label* m_pLabel_Name;
   Gtk::Label* m_pLabel_Table;
   Gtk::Label* m_pLabel_Mode;
   Gtk::Label* m_pLabel_userlevel;
