@@ -47,24 +47,24 @@ Box_Data::~Box_Data()
   }
 }
 
-void Box_Data::init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause)
+bool Box_Data::init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause)
 {
   m_strTableName = strTableName;
   m_strWhereClause = strWhereClause;
 
   create_layout(); //So that fill_from_database() can succeed.
 
-  Box_DB_Table::init_db_details(strTableName); //Calls fill_from_database().
+  return Box_DB_Table::init_db_details(strTableName); //Calls fill_from_database().
 }
 
-void Box_Data::refresh_data_from_database(const Glib::ustring& strWhereClause)
+bool Box_Data::refresh_data_from_database(const Glib::ustring& strWhereClause)
 {
   m_strWhereClause = strWhereClause;
 
-  Box_DB_Table::refresh_data_from_database(); //Calls fill_from_database().
+  return Box_DB_Table::refresh_data_from_database(); //Calls fill_from_database().
 }
 
-Glib::ustring Box_Data::get_WhereClause() const
+Glib::ustring Box_Data::get_find_where_clause() const
 {
   Glib::ustring strClause;
 
@@ -88,8 +88,7 @@ Glib::ustring Box_Data::get_WhereClause() const
 
       if(use_this_field)
       {
-        //TODO: "table.name"
-        strClausePart = field.get_name() + " LIKE " +  field.sql(data); //% is mysql wildcard for 0 or more characters.
+        strClausePart = m_strTableName + "." + field.get_name() + " " + field.sql_find_operator() + " " +  field.sql(data); //% is mysql wildcard for 0 or more characters.
       }
     }
 
@@ -105,7 +104,7 @@ void Box_Data::on_Button_Find()
   //Make sure that the cell is updated:
   //m_AddDel.finish_editing();
 
-  signal_find.emit(get_WhereClause());
+  signal_find_criteria.emit(get_find_where_clause());
 }
 
 Box_Data::type_map_fields Box_Data::get_record_field_values(const Gnome::Gda::Value& primary_key_value)
@@ -292,11 +291,11 @@ void Box_Data::create_layout()
   m_TableFields = get_fields_for_table(m_strTableName);
 }
 
-void Box_Data::fill_from_database()
+bool Box_Data::fill_from_database()
 {
   set_unstored_data(false);
 
-  Box_DB_Table::fill_from_database();
+  return Box_DB_Table::fill_from_database();
 }
 
 bool Box_Data::confirm_discard_unstored_data() const
@@ -517,6 +516,11 @@ Box_Data::type_vecLayoutFields Box_Data::get_table_fields_to_show(const Glib::us
 
 Gnome::Gda::Value Box_Data::generate_next_auto_increment(const Glib::ustring& table_name, const Glib::ustring field_name)
 {
+  //Get it from the database system table.
+  //The developer can change the next value in the Database Preferences
+  return get_next_auto_increment_value(table_name, field_name);
+
+  /*
   //This is a workaround for postgres problems. Ideally, we need to use the postgres serial type and find out how to get the generated value after we add a row.
 
   double result = 0;
@@ -540,6 +544,7 @@ Gnome::Gda::Value Box_Data::generate_next_auto_increment(const Glib::ustring& ta
   //Get a string representation of the number, so we can put it back in a NUMERIC Gda::Value:
 
   return GlomConversions::parse_value(result);
+  */
 }
 
 /** Get the shown fields that are in related tables, via a relationship using @a field_name changes.
