@@ -36,6 +36,7 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   m_button_field_add_group(0),
   m_button_add_related(0),
   m_button_field_delete(0),
+  m_button_field_formatting(0),
   m_button_field_edit(0),
   m_label_table_name(0)
 {
@@ -112,6 +113,9 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   refGlade->get_widget("button_field_delete", m_button_field_delete);
   m_button_field_delete->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_field_delete) );
 
+  refGlade->get_widget("button_formatting", m_button_field_formatting);
+  m_button_field_formatting->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_field_formatting) );
+
   refGlade->get_widget("button_field_add", m_button_field_add);
   m_button_field_add->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_field_add) );
 
@@ -162,7 +166,8 @@ void Dialog_Layout_Details::fill_group(const Gtk::TreeModel::iterator& iter, Lay
       else if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_FIELD)
       {
         //Add field:
-        LayoutItem_Field field;
+        LayoutItem_Field field = rowChild[m_model_items->m_columns.m_col_field_formatting]; //TODO: Use _only_ this for fields in future?
+
         field.set_name( rowChild[m_model_items->m_columns.m_col_name] );
 
         field.m_relationship = rowChild[m_model_items->m_columns.m_col_relationship_name];
@@ -202,6 +207,7 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
     row[m_model_items->m_columns.m_col_name] = group.get_name();
     row[m_model_items->m_columns.m_col_columns_count] = group.m_columns_count;
     row[m_model_items->m_columns.m_col_title] = group.m_title;
+    row[m_model_items->m_columns.m_col_editable] = group.get_editable();
 
     //Add the child items:
     LayoutGroup::type_map_const_items items = group.get_items();
@@ -230,6 +236,7 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
             Gtk::TreeModel::iterator iterField = m_model_items->append(iterNewGroup->children());
             Gtk::TreeModel::Row row = *iterField;
             row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_FIELD;
+            row[m_model_items->m_columns.m_col_field_formatting] = *field;
             row[m_model_items->m_columns.m_col_name] = field->get_name();
             row[m_model_items->m_columns.m_col_relationship_name] = field->m_relationship;
 
@@ -333,6 +340,8 @@ void Dialog_Layout_Details::enable_buttons()
       m_button_field_down->set_sensitive(enable_down);
 
       m_button_field_delete->set_sensitive(true);
+
+      m_button_field_formatting->set_sensitive( (*iter)[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_FIELD);
     }
     else
     {
@@ -340,6 +349,7 @@ void Dialog_Layout_Details::enable_buttons()
       m_button_field_down->set_sensitive(false);
       m_button_field_up->set_sensitive(false);
       m_button_field_delete->set_sensitive(false);
+      m_button_field_formatting->set_sensitive(false);
     }
   }
 
@@ -455,6 +465,7 @@ void Dialog_Layout_Details::on_button_field_add()
       row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_FIELD;
       row[m_model_items->m_columns.m_col_name] = layout_item.get_name();
       row[m_model_items->m_columns.m_col_relationship_name] = layout_item.m_relationship;
+      row[m_model_items->m_columns.m_col_editable] = true; //A sane default.
       //row[m_model_items->m_columns.m_col_title] = field.get_title();
 
       //Scroll to, and select, the new row:
@@ -465,6 +476,8 @@ void Dialog_Layout_Details::on_button_field_add()
       m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
     }
   }
+
+  enable_buttons();
 }
 
 
@@ -481,7 +494,7 @@ bool Dialog_Layout_Details::offer_relationship_list(Relationship& relationship)
 
     if(dialog)
     {
-      dialog->set_document(m_document, m_table_name);
+      dialog->set_document(get_document(), m_table_name);
       dialog->set_transient_for(*this);
       int response = dialog->run();
       dialog->hide();
@@ -515,7 +528,7 @@ bool Dialog_Layout_Details::offer_field_list(LayoutItem_Field& field)
 
     if(dialog)
     {
-      dialog->set_document(m_document, m_table_name, field);
+      dialog->set_document(get_document(), m_table_name, field);
       dialog->set_transient_for(*this);
       int response = dialog->run();
       if(response == Gtk::RESPONSE_OK)
@@ -548,7 +561,8 @@ bool Dialog_Layout_Details::offer_field_layout(LayoutItem_Field& field)
 
     if(dialog)
     {
-      dialog->set_field(field);
+      add_view(dialog); //Give it access to the document.
+      dialog->set_field(field, m_table_name);
       dialog->set_transient_for(*this);
       int response = dialog->run();
       if(response == Gtk::RESPONSE_OK)
@@ -557,6 +571,7 @@ bool Dialog_Layout_Details::offer_field_layout(LayoutItem_Field& field)
         result = dialog->get_field_chosen(field);
       }
 
+      remove_view(dialog);
       delete dialog;
     }
   }
@@ -597,6 +612,7 @@ void Dialog_Layout_Details::on_button_add_related()
       Gtk::TreeModel::Row row = *iter;
       row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_PORTAL;
       row[m_model_items->m_columns.m_col_relationship] = relationship.get_name();
+      row[m_model_items->m_columns.m_col_editable] = true; //A sane default.
       //row[m_model_items->m_columns.m_col_title] = field.get_title();
 
       //Scroll to, and select, the new row:
@@ -607,6 +623,8 @@ void Dialog_Layout_Details::on_button_add_related()
       m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
     }
   }
+
+  enable_buttons();
 }
 
 Gtk::TreeModel::iterator Dialog_Layout_Details::get_selected_group_parent() const
@@ -659,6 +677,7 @@ void Dialog_Layout_Details::on_button_field_add_group()
     row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_GROUP;
     row[m_model_items->m_columns.m_col_name] = "Untitled group";
     row[m_model_items->m_columns.m_col_columns_count] = 1;
+    row[m_model_items->m_columns.m_col_editable] = true; //A sane default.
 
     //Scroll to, and select, the new row:
     Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
@@ -669,6 +688,58 @@ void Dialog_Layout_Details::on_button_field_add_group()
 
     m_modified = true;
   }
+
+  enable_buttons();
+}
+
+void Dialog_Layout_Details::on_button_field_formatting()
+{
+  //TODO: Abstract this into the base class:
+
+  //Get the selected item:
+  Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
+  if(refTreeSelection)
+  {
+    Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
+    if(iter)
+    {
+      Gtk::TreeModel::Row row = *iter;
+
+      try
+      {
+        Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_layout_field_properties");
+
+        Dialog_FieldLayout* dialog = 0;
+        refXml->get_widget_derived("dialog_layout_field_properties", dialog);
+
+        if(dialog)
+        {
+          add_view(dialog); //Give it access to the document.
+
+          LayoutItem_Field field = row[m_model_items->m_columns.m_col_field_formatting];
+
+          dialog->set_field(field, m_table_name);
+          dialog->set_transient_for(*this);
+          int response = dialog->run();
+          dialog->hide();
+          if(response == Gtk::RESPONSE_OK)
+          {
+            //Get the chosen field:
+            bool test = dialog->get_field_chosen(field);
+            if(test)
+              row[m_model_items->m_columns.m_col_field_formatting] = field;
+          }
+
+          remove_view(dialog);
+          delete dialog;
+        }
+      }
+      catch(const Gnome::Glade::XmlError& ex)
+      {
+        std::cerr << ex.what() << std::endl;
+      }
+    }
+  }
 }
 
 void Dialog_Layout_Details::on_button_field_edit()
@@ -738,8 +809,9 @@ void Dialog_Layout_Details::save_to_document()
   if(m_modified)
   {
     //Set the table name and title:
-    if(m_document)
-      m_document->set_table_title( m_table_name, m_entry_table_title->get_text() );
+    Document_Glom* document = get_document();
+    if(document)
+      document->set_table_title( m_table_name, m_entry_table_title->get_text() );
 
     //Get the data from the TreeView and store it in the document:
 
@@ -767,9 +839,9 @@ void Dialog_Layout_Details::save_to_document()
       }
     }
 
-    if(m_document)
+    if(document)
     {
-      m_document->set_data_layout_groups(m_layout_name, m_table_name, mapGroups);
+      document->set_data_layout_groups(m_layout_name, m_table_name, mapGroups);
       m_modified = false;
     }
   }
