@@ -24,7 +24,7 @@
 Box_Data_Details_Find::Box_Data_Details_Find()
 : Box_Data_Details(false)
 {
-   m_strHint = _("Enter the search criteria and click [Find]\n Glom will then change to Data mode to display the results.");
+   //m_strHint = _("Enter the search criteria and click [Find]\n Glom will then change to Data mode to display the results.");
 
   //Instead of nav buttons:
   m_HBox.pack_end(m_Button_Find, Gtk::PACK_SHRINK);
@@ -40,15 +40,17 @@ Box_Data_Details_Find::~Box_Data_Details_Find()
 
 
 
-void Box_Data_Details_Find::fill_from_database()
+bool Box_Data_Details_Find::fill_from_database()
 {
   Bakery::BusyCursor(*get_app_window());
 
-  Box_DB_Table::fill_from_database();
+  bool result = Box_DB_Table::fill_from_database();
 
-  m_Fields = get_fields_to_show();
+  m_FieldsShown = get_fields_to_show();
 
-  fill_from_database_layout(); //TODO: Only do this when the layout has changed.
+  create_layout(); //TODO: Only do this when the layout has changed.
+
+  return result;
 }
 
 void Box_Data_Details_Find::fill_related()
@@ -57,7 +59,7 @@ void Box_Data_Details_Find::fill_related()
   //m_Notebook_Related.pages().clear();
 
   //Get relationships from the document:
-  Document_Glom::type_vecRelationships vecRelationships = m_pDocument->get_relationships(m_strTableName);
+  Document_Glom::type_vecRelationships vecRelationships = get_document()->get_relationships(m_strTableName);
 
   //Add the relationships:
   for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
@@ -71,9 +73,9 @@ void Box_Data_Details_Find::fill_related()
      m_Notebook_Related.pages().push_back( Gtk::Notebook_Helpers::TabElem(*pBox, relationship.get_name()) );
      std::cout <<  "Box_Data_Details::fill_related() 2.5:" << std::endl;
 
-     guint rowKey = m_Fields.get_index(relationship.get_from_field());
+     guint rowKey = m_FieldsShown.get_index(relationship.get_from_field());
      Glib::ustring strKeyValue = m_AddDel.get_value(rowKey);
-     strKeyValue = m_Fields[rowKey].sql(strKeyValue); //Quote/Escape it if necessary.
+     strKeyValue = m_FieldsShown[rowKey].sql(strKeyValue); //Quote/Escape it if necessary.
 
      std::cout <<  "Box_Data_Details::fill_related() 3:" << std::endl;
      pBox->init_db_details(get_database_name(), relationship.get_to_table(), relationship.get_to_field(), strKeyValue);
