@@ -23,22 +23,20 @@
 
 #include "../mode_data/box_data_details.h"
 
-/**
-  *@author Murray Cumming
-  */
-
 class Box_Data_Details_Find : public Box_Data_Details
 {
 public: 
-	Box_Data_Details_Find();
-	virtual ~Box_Data_Details_Find();
+  Box_Data_Details_Find();
+  virtual ~Box_Data_Details_Find();
 
+  virtual Gtk::Widget* get_default_button(); //override
+    
 protected:
 
-  virtual void fill_from_database(); //override.
+  virtual bool fill_from_database(); //override.
   virtual void fill_related(); //override.
 
-  virtual void on_AddDel_user_changed(guint row, guint col); //override
+  virtual void on_adddel_user_changed(guint row, guint col); //override
 
 
 };
