@@ -34,7 +34,7 @@ public:
   virtual ~Box_Tables();
 
 protected:
-  virtual void fill_from_database(); //override
+  virtual bool fill_from_database(); //override
 
   virtual void fill_table_row(const Gtk::TreeModel::iterator& iter, const TableInfo& table_info);
 
