@@ -73,7 +73,9 @@ DbAddDel::DbAddDel()
   //set_columns_count(1);
   //construct_specified_columns();
 
+  m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
   m_ScrolledWindow.add(m_TreeView);
+
   m_TreeView.show();
   //m_TreeView.set_fixed_height_mode(); //This allows some optimizations.
   pack_start(m_ScrolledWindow);
@@ -104,12 +106,6 @@ DbAddDel::DbAddDel()
 
 DbAddDel::~DbAddDel()
 {
-  //if(m_modelcolumn_key)
-  //  delete m_modelcolumn_key;
-
-  //if(m_modelcolumn_placeholder)
-  //  delete m_modelcolumn_placeholder;
-
   App_Glom* pApp = get_application();
   if(pApp)
   {
@@ -346,7 +342,6 @@ Gtk::TreeModel::iterator DbAddDel::add_item(const Gnome::Gda::Value& valKey)
     Gtk::TreeModel::Row treerow = *result;
     if(treerow)
     {
-      treerow[*m_modelcolumn_key] = valKey;
       set_value_key(result, valKey);
       m_refListStore->set_is_placeholder(result, false);
       //treerow[*m_modelcolumn_placeholder] =  false; 
@@ -626,6 +621,7 @@ void DbAddDel::construct_specified_columns()
   }
 
   //Create the model from the ColumnRecord:
+  g_warning("DbAddDel::construct_specified_columns(): where_clause=%s", m_where_clause.c_str());
   m_refListStore = type_model_store::create(record, m_table_name, fields, column_index_key, m_allow_view, m_where_clause);
 
   m_TreeView.set_model(m_refListStore);
@@ -787,6 +783,20 @@ void DbAddDel::construct_specified_columns()
   add_blank();
 }
 
+bool DbAddDel::refresh_from_database()
+{
+  if(m_refListStore)
+  {
+    Glib::RefPtr<Gtk::TreeModel> refNull;
+    bool result = m_refListStore->refresh_from_database(m_where_clause);
+    m_TreeView.set_model(refNull);
+    m_TreeView.set_model(m_refListStore);
+    return result;
+  }
+  else
+    return false;
+}
+
 void DbAddDel::set_value(const Gtk::TreeModel::iterator& iter, const LayoutItem_Field& layout_item, const Gnome::Gda::Value& value)
 {
   //g_warning("DbAddDel::set_value begin");
@@ -1563,8 +1573,6 @@ void DbAddDel::set_value_key(const Gtk::TreeModel::iterator& iter, const Gnome::
     }
 
     m_refListStore->set_key_value(iter, value);
-
-   // row[*m_modelcolumn_key] = value;
   }
 }
 
