@@ -41,6 +41,17 @@ FlowTableWithFields::FlowTableWithFields(const Glib::ustring& table_name)
 
 FlowTableWithFields::~FlowTableWithFields()
 {
+  //Remove views. The widgets are deleted automatically because they are managed.
+  for(type_listFields::iterator iter = m_listFields.begin(); iter != m_listFields.end(); ++iter)
+  {
+    View_Composite_Glom* pViewFirst = dynamic_cast<View_Composite_Glom*>(iter->m_first);
+    if(pViewFirst)
+      remove_view(pViewFirst);
+
+   View_Composite_Glom* pViewSecond = dynamic_cast<View_Composite_Glom*>(iter->m_second);
+    if(pViewSecond)
+      remove_view(pViewSecond);
+  }
 }
 
 void FlowTableWithFields::set_table(const Glib::ustring& table_name)
@@ -79,7 +90,7 @@ void FlowTableWithFields::add_layout_item_at_position(const LayoutItem& item, co
       add_field_at_position(*field, m_table_name, add_before);
 
       //Do not allow editing of auto-increment fields:
-      if(field->m_field.get_field_info().get_auto_increment())
+      if(field->m_field.get_auto_increment())
         set_field_editable(*field, false);
       else
         set_field_editable(*field, field->get_editable_and_allowed());
@@ -97,8 +108,9 @@ void FlowTableWithFields::add_layout_item_at_position(const LayoutItem& item, co
           if(found)
           {
             Box_Data_List_Related* portal_box = Gtk::manage(new Box_Data_List_Related);
+            add_view(portal_box); //Give it access to the document, needed to get the layout and fields information.
 
-            portal_box->init_db_details(relationship);
+            portal_box->init_db_details(relationship); //Create the layout
 
             portal_box->set_layout_item(portal->clone(), relationship.get_to_table());
             portal_box->show();
@@ -109,8 +121,6 @@ void FlowTableWithFields::add_layout_item_at_position(const LayoutItem& item, co
 
             //Connect signals:
             portal_box->signal_record_changed().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_portal_record_changed) );
-
-            add_view(portal_box);
           }
         }
       }
@@ -243,7 +253,7 @@ void FlowTableWithFields::add_field_at_position(const LayoutItem_Field& layoutit
   info.m_field = layoutitem_field;
 
   //Add the entry or checkbox (handled by the DataWidget)
-  DataWidget* pDataWidget = Gtk::manage(new DataWidget(layoutitem_field, table_name) );
+  DataWidget* pDataWidget = Gtk::manage(new DataWidget(layoutitem_field, table_name, get_document()) );
   add_layoutwidgetbase(pDataWidget, add_before);
   add_view(pDataWidget); //So it can get the document.
 
@@ -267,7 +277,12 @@ void FlowTableWithFields::add_field_at_position(const LayoutItem_Field& layoutit
 
   //info.m_group = layoutitem_field.m_group;
 
-  add(*(info.m_first), *(info.m_second));
+  //Expand multiline text fields to take up the maximum possible width:
+  bool expand_second = false;
+  if( (layoutitem_field.m_field.get_glom_type() == Field::TYPE_TEXT) && layoutitem_field.get_text_format_multiline())
+    expand_second = true;
+
+  add(*(info.m_first), *(info.m_second), expand_second);
 
   info.m_second->signal_edited().connect( sigc::bind(sigc::mem_fun(*this, &FlowTableWithFields::on_entry_edited), layoutitem_field)  ); //TODO:  Is it a good idea to bind the LayoutItem? sigc::bind() probably stores a copy at this point.
   info.m_second->signal_layout_item_added().connect( sigc::bind(
@@ -285,7 +300,16 @@ void FlowTableWithFields::remove_field(const Glib::ustring& id)
       Info info = *iter;
       remove(*(info.m_first));
 
+      View_Composite_Glom* pViewFirst = dynamic_cast<View_Composite_Glom*>(info.m_first);
+      if(pViewFirst)
+        remove_view(pViewFirst);
+
       delete info.m_first;
+
+      View_Composite_Glom* pViewSecond = dynamic_cast<View_Composite_Glom*>(info.m_second);
+      if(pViewSecond)
+        remove_view(pViewSecond);
+
       delete info.m_second; //It is removed at the same time.
 
       iter = m_listFields.erase(iter);
@@ -314,7 +338,7 @@ void FlowTableWithFields::set_field_value(const LayoutItem_Field& field, const G
     if(portal)
     {
       //g_warning("FlowTableWithFields::set_field_value: foreign_key_value=%s", value.to_string().c_str());
-      portal->refresh_db_details(value /* foreign key value */, Gnome::Gda::Value() /* TODO */);
+      portal->refresh_data_from_database(value /* foreign key value */);
     }
   }
 }
@@ -464,7 +488,6 @@ void FlowTableWithFields::change_group(const Glib::ustring& /* id */, const Glib
 
 void FlowTableWithFields::remove_all()
 {
-  //TODO: Release the fields memory, and the portal memory.
   m_listFields.clear();
 
   for(type_sub_flow_tables::iterator iter = m_sub_flow_tables.begin(); iter != m_sub_flow_tables.end(); ++iter)
@@ -493,6 +516,18 @@ void FlowTableWithFields::remove_all()
 
   m_list_layoutwidgets.clear();
 
+  //Remove views. The widgets are deleted automatically because they are managed.
+  for(type_listFields::iterator iter = m_listFields.begin(); iter != m_listFields.end(); ++iter)
+  {
+    View_Composite_Glom* pViewFirst = dynamic_cast<View_Composite_Glom*>(iter->m_first);
+    if(pViewFirst)
+      remove_view(pViewFirst);
+
+   View_Composite_Glom* pViewSecond = dynamic_cast<View_Composite_Glom*>(iter->m_second);
+    if(pViewSecond)
+      remove_view(pViewSecond);
+  }
+
   FlowTable::remove_all();
 }
 
@@ -583,6 +618,7 @@ void FlowTableWithFields::get_layout_group(LayoutGroup& group)
       else
       {
         const LayoutItem* pLayoutItem = (*iter)->get_layout_item();
+
         if(pLayoutItem)
           group.add_item(*pLayoutItem);
       }
